#ifndef __NETWORK_H__
#define __NETWORK_H__

#include <stdio.h>
#include <stdlib.h>
#include <sys/ksys.h>

#include "childWindow.hpp"

/// \brief Основное пространство имён
/// \author Egor00f
namespace KolibriLib
{
    
    /// @brief Работа с сетью
    namespace Network
    {

        /// @brief Работа с сетевой картой
        namespace Devices
        {
            /// @brief Получить кол-во сетевых устройств
            /// @return Кол-во сетевых устройств
            int GetDevicesAmount()
            {
                int amount;
                asm_inline(
                    "int $0x40"
                    :"=a"(amount)
                    :"a"(74), "b"(-1)
                );
                return amount;
            }

            /// @brief Получить тип устройства
            /// @param device устройство
            /// @return тип устройства
            int GetDeviceType(int device)
            {
                int type;
                asm_inline(
                    "int $0x40"
                    :"=a"(type)
                    :"a"(74), "b"(0)
                );
                return type;
            }

            /// @brief Получить имя устройства
            /// @param Device устройство
            /// @return Имя устройства
            std::string GetDeviceName(int Device)
            {
                int abc;
                void *buff = malloc(64);
                asm_inline(
                    "int $0x40"
                    :"=a"(abc)
                    :"a"(74), "b"((1 << 16) + Device), "c"(buff)
                );
                return std::string((char*)buff);
            }

            bool ResetDevice(unsigned short Device)
            {
                int error;
                void *ptr = malloc(64);
                asm_inline(
                    "int $0x40"
                    :"=a"(error)
                    :"a"(74), "b"((2 << 16) + Device), "c"(ptr)
                );
                return error;
            } 
        } // namespace Devices
        

        /// @brief Работа с сокетами
        namespace Socket
        {
            #include <sys/socket.h>

            /// @brief 
            enum Domain
            {
                IPv4 = 2,
                IPv6 = 10
            };
            /// @brief Список типов сокета
            enum SocetTypes
            {
                Stream = 1,
                DGram = 2,
                RAW = 3
            };

            /// @brief Список протоколов
            enum Protocols
            {
                PROTOCOL_IP = 0,
                PROTOCOL_CMP = 1,
                PROTOCOL_TCP = 6,
                PROTOCOL_UDP = 17,
                PROTOCOL_RAW = 255
            };

            /// @brief Окрыть сокет
            /// @param domain тип сети IPv4/IPv6
            /// @param SocetType 
            /// @param protocol Используемый протокол
            /// @return номер сокета
            inline int OpenSocket(int domain = IPv4, int SocetType = Stream, int protocol = PROTOCOL_IP)
            {
                int Socket = socket(domain, SocetType, protocol);

                if(err_code == -1)
                {
                    childWindow::ErrorWindow("He yдaлocь открыть сокет");
                }

                return Socket;
            }

            /// @brief Закрыть сокет
            /// @param Socket номер сокета
            /// @return true если удачно, иначе false
            bool CloseSocket(int Socket)
            {
                int a = close(Socket);
                if(a == -1)
                {
                    return false;
                }
                return true;
            }

            bool Bind(int Socket)
            {
                //bind(Socket, );
            }

        } // namespace Socket
        
    } // namespace network
    
} // namespace KolibriLib


#endif // __NETWORK_H__